From ffef2e1e98cff449218dcb70ae8d6928774deacf Mon Sep 17 00:00:00 2001
From: Miao Wang <shankerwangmiao@gmail.com>
Date: Wed, 21 Feb 2018 21:34:00 +0800
Subject: [PATCH] Retain CAP_NET_BIND_SERVICE capability for udp priviliged
 bind.

The capability is retained automatically in unprivileged worker
processes after changing UID if at least one stream server
listens on a port below 1024 in nginx configuration.

The feature is only available in Linux.
---
 src/core/ngx_cycle.h                |  1 +
 src/os/unix/ngx_process_cycle.c     | 12 +++++++++---
 src/stream/ngx_stream_core_module.c |  9 +++++++++
 3 files changed, 19 insertions(+), 3 deletions(-)

diff --git a/src/core/ngx_cycle.h b/src/core/ngx_cycle.h
index 54fa2e6b..a1ea0b4c 100644
--- a/src/core/ngx_cycle.h
+++ b/src/core/ngx_cycle.h
@@ -117,6 +117,7 @@ typedef struct {
     char                    **environment;
 
     ngx_uint_t                transparent;  /* unsigned  transparent:1; */
+    ngx_uint_t                need_cap_bind; /* unsigned  need:1; */
 } ngx_core_conf_t;
 
 
diff --git a/src/os/unix/ngx_process_cycle.c b/src/os/unix/ngx_process_cycle.c
index 5817a2c2..5548f35d 100644
--- a/src/os/unix/ngx_process_cycle.c
+++ b/src/os/unix/ngx_process_cycle.c
@@ -840,7 +840,7 @@ ngx_worker_process_init(ngx_cycle_t *cycle, ngx_int_t worker)
         }
 
 #if (NGX_HAVE_PR_SET_KEEPCAPS && NGX_HAVE_CAPABILITIES)
-        if (ccf->transparent && ccf->user) {
+        if ((ccf->need_cap_bind || ccf->transparent) && ccf->user) {
             if (prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0) == -1) {
                 ngx_log_error(NGX_LOG_EMERG, cycle->log, ngx_errno,
                               "prctl(PR_SET_KEEPCAPS, 1) failed");
@@ -858,7 +858,7 @@ ngx_worker_process_init(ngx_cycle_t *cycle, ngx_int_t worker)
         }
 
 #if (NGX_HAVE_CAPABILITIES)
-        if (ccf->transparent && ccf->user) {
+        if ((ccf->need_cap_bind || ccf->transparent) && ccf->user) {
             struct __user_cap_data_struct    data;
             struct __user_cap_header_struct  header;
 
@@ -866,7 +866,13 @@ ngx_worker_process_init(ngx_cycle_t *cycle, ngx_int_t worker)
             ngx_memzero(&data, sizeof(struct __user_cap_data_struct));
 
             header.version = _LINUX_CAPABILITY_VERSION_1;
-            data.effective = CAP_TO_MASK(CAP_NET_RAW);
+            data.effective = 0;
+            if (ccf->transparent) {
+              data.effective |= CAP_TO_MASK(CAP_NET_RAW);
+            }
+            if (ccf->need_cap_bind) {
+              data.effective |= CAP_TO_MASK(CAP_NET_BIND_SERVICE);
+            }
             data.permitted = data.effective;
 
             if (syscall(SYS_capset, &header, &data) == -1) {
diff --git a/src/stream/ngx_stream_core_module.c b/src/stream/ngx_stream_core_module.c
index d98e40ae..814fa86e 100644
--- a/src/stream/ngx_stream_core_module.c
+++ b/src/stream/ngx_stream_core_module.c
@@ -624,6 +624,15 @@ ngx_stream_core_listen(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 #if !(NGX_WIN32)
         if (ngx_strcmp(value[i].data, "udp") == 0) {
             ls->type = SOCK_DGRAM;
+            if (ntohs(u.sockaddr.sockaddr_in.sin_port) < 1024){
+              // here we assume af_inet socket, since the sin(6)_port fields are at 
+              // the same offset
+              ngx_core_conf_t  *ccf;
+              
+              ccf = (ngx_core_conf_t *) ngx_get_conf(cf->cycle->conf_ctx,
+                                                        ngx_core_module);
+              ccf->need_cap_bind = 1;
+            }
             continue;
         }
 #endif
-- 
2.11.0

